@@ -1,17 +1,13 @@
 //! Persistent, ACID compliant, threadsafe to-disk store.
 //! Powered by Sled - an embedded database.
 
-use std::{
-    collections::HashMap,
-    sync::{Arc, Mutex},
-};
+use std::{collections::{HashMap, HashSet}, sync::{Arc, Mutex}};
 
-use crate::{
-    errors::AtomicResult,
-    resources::PropVals,
-    storelike::{ResourceCollection, Storelike},
-    Atom, Resource,
-};
+use crate::{Atom, Resource, Value, datatype::DataType, errors::AtomicResult, resources::PropVals, storelike::{ResourceCollection, Storelike}};
+
+/// Inside the index_vals, each value is mapped to this type.
+/// The String on the left represents a Property URL, and the second one is the set of subjects.
+pub type PropSubjectMap = HashMap<String, HashSet<String>>;
 
 /// The Db is a persistent on-disk Atomic Data store.
 /// It's an implementation of Storelike.
@@ -26,7 +22,6 @@ pub struct Db {
     resources: sled::Tree,
     // Stores all Atoms. The key is the atom.value, the value a vector of Atoms.
     index_vals: sled::Tree,
-    index_props: sled::Tree,
     /// The base_url is the domain where the db will be hosted, e.g. http://localhost/
     base_url: String,
 }
@@ -36,30 +31,20 @@ impl Db {
     /// The base_url is the domain where the db will be hosted, e.g. http://localhost/
     /// It is used for distinguishing locally defined items from externally defined ones.
     pub fn init<P: AsRef<std::path::Path>>(path: P, base_url: String) -> AtomicResult<Db> {
-        let db = sled::open(path).map_err(|e|format!("Failed creating DB at this location. Is another instance of Atomic Server running? {}", e))?;
+        let db = sled::open(path).map_err(|e|format!("Failed opening DB at this location. Is another instance of Atomic Server running? {}", e))?;
         let resources = db.open_tree("resources").map_err(|e|format!("Failed building resources. Your DB might be corrupt. Go back to a previous version and export your data. {}", e))?;
-        let index_props = db.open_tree("index_props")?;
         let index_vals = db.open_tree("index_vals")?;
         let store = Db {
             db,
             default_agent: Arc::new(Mutex::new(None)),
             resources,
             index_vals,
-            index_props,
             base_url,
         };
         crate::populate::populate_base_models(&store)?;
         Ok(store)
     }
 
-    // fn index_value_add(&mut self, atom: Atom) -> AtomicResult<()> {
-    //     todo!();
-    // }
-
-    // fn index_value_remove(&mut self, atom: Atom) -> AtomicResult<()> {
-    //     todo!();
-    // }
-
     /// Internal method for fetching Resource data.
     fn set_propvals(&self, subject: &str, propvals: &PropVals) -> AtomicResult<()> {
         let resource_bin = bincode::serialize(propvals)?;
@@ -80,12 +65,49 @@ impl Db {
         match propval_maybe.as_ref() {
             Some(binpropval) => {
                 let propval: PropVals = bincode::deserialize(binpropval)
-                    .map_err(|e| format!("{} {}", corrupt_db_message(subject), e))?;
+                    .map_err(|e| format!("Deserialize propval error: {} {}", corrupt_db_message(subject), e))?;
                 Ok(propval)
             }
             None => Err(format!("Resource {} not found", subject).into()),
         }
     }
+
+    /// Search for a value, get a PropSubjectMap. If it does not exist, create a new one.
+    fn get_prop_subject_map(&self, string_val: &str) -> AtomicResult<PropSubjectMap> {
+        let prop_sub_map = self
+            .index_vals
+            .get(string_val)
+            .map_err(|e| format!("Can't open {} from value index: {}", string_val, e))?;
+        match prop_sub_map.as_ref() {
+                Some(binpropval) => {
+                    let psm: PropSubjectMap = bincode::deserialize(binpropval)
+                        .map_err(|e| format!("Deserialize PropSubjectMap error: {} {}", corrupt_db_message(&string_val), e))?;
+                Ok(psm)
+            }
+            None => {
+                let psm: PropSubjectMap = PropSubjectMap::new();
+                Ok(psm)
+            },
+        }
+    }
+
+    /// Returns true if the index has been built.
+    pub fn has_index(&self) -> bool {
+        !self.index_vals.is_empty()
+    }
+
+    fn set_prop_subject_map(&self, string_val: &str, psm: &PropSubjectMap) -> AtomicResult<()> {
+        let psm_binary = bincode::serialize(psm)
+        .map_err(|e| format!("Can't serialize value {}: {}", string_val, e))?;
+        self.index_vals.insert(string_val, psm_binary)?;
+        Ok(())
+    }
+
+    /// Removes all values from the index.
+    pub fn clear_index(&self) -> AtomicResult<()> {
+        self.index_vals.clear()?;
+        Ok(())
+    }
 }
 
 impl Storelike for Db {
@@ -117,6 +139,33 @@ impl Storelike for Db {
         Ok(())
     }
 
+    // This only adds ResourceArrays and AtomicURLs at this moment, which means that many values cannot be accessed in the TPF query (thus, collections)
+    fn add_atom_to_index(&self, atom: &Atom) -> AtomicResult<()> {
+
+        let vec = match atom.value.clone() {
+            Value::ResourceArray(v) => v,
+            Value::AtomicUrl(v) => vec![v],
+            _other=>  {
+                return Ok(())
+            },
+        };
+
+        for val in vec {
+            let mut map = self.get_prop_subject_map(&val)?;
+
+            let mut set = match map.get_mut(&atom.property){
+                Some(vals) => vals.to_owned(),
+                None => HashSet::new(),
+            };
+
+            set.insert(atom.subject.clone());
+            map.insert(atom.property.clone(), set);
+
+            self.set_prop_subject_map(&val, &map)?;
+        }
+        Ok(())
+    }
+
     fn add_resource(&self, resource: &Resource) -> AtomicResult<()> {
         // This only works if no external functions rely on using add_resource for atom-like operations!
         // However, add_atom uses set_propvals, which skips the validation.
@@ -128,6 +177,28 @@ impl Storelike for Db {
         self.set_propvals(resource.get_subject(), &resource.get_propvals())
     }
 
+    fn remove_atom_from_index(&self, atom: &Atom) -> AtomicResult<()> {
+        let vec = match atom.value.clone() {
+            Value::ResourceArray(v) => v,
+            other=>  vec![other.to_string()],
+        };
+
+        for val in vec {
+            let mut map = self.get_prop_subject_map(&val)?;
+
+            let mut set = match map.get_mut(&atom.property){
+                Some(vals) => vals.to_owned(),
+                None => HashSet::new(),
+            };
+
+            set.remove(&atom.subject);
+            map.insert(atom.property.clone(), set);
+
+            self.set_prop_subject_map(&val, &map)?;
+        }
+        Ok(())
+    }
+
     fn get_base_url(&self) -> &str {
         &self.base_url
     }
@@ -226,9 +297,15 @@ impl Storelike for Db {
     fn populate(&self) -> AtomicResult<()> {
         // populate_base_models should be run in init, instead of here, since it will result in infinite loops without
         crate::populate::populate_default_store(self)?;
+        // This is a potentially expensive operation, but is needed to make TPF queries work with the models created in here
+        self.build_index(true)?;
         crate::populate::populate_hierarchy(self)?;
         crate::populate::populate_collections(self)?;
-        crate::populate::populate_endpoints(self)
+        crate::populate::populate_endpoints(self)?;
+        // We need to build the index again, because the the endpoints and collections might not be indexed.
+        // Ideally, the populate methods themselves make sure they are indexed.
+        // self.build_index(true)?;
+        Ok(())
     }
 
     fn remove_resource(&self, subject: &str) -> AtomicResult<()> {
@@ -249,6 +326,123 @@ impl Storelike for Db {
     fn set_default_agent(&self, agent: crate::agents::Agent) {
         self.default_agent.lock().unwrap().replace(agent);
     }
+
+    // TPF implementation that used the index_value cache, far more performant than the StoreLike implementation
+    fn tpf(
+        &self,
+        q_subject: Option<&str>,
+        q_property: Option<&str>,
+        q_value: Option<&str>,
+        // Whether resources from outside the store should be searched through
+        include_external: bool,
+    ) -> AtomicResult<Vec<Atom>> {
+        let mut vec: Vec<Atom> = Vec::new();
+
+        let hassub = q_subject.is_some();
+        let hasprop = q_property.is_some();
+        let hasval = q_value.is_some();
+
+        // Simply return all the atoms
+        if !hassub && !hasprop && !hasval {
+            for resource in self.all_resources(include_external) {
+                for (property, value) in resource.get_propvals() {
+                    vec.push(Atom::new(
+                        resource.get_subject().clone(),
+                        property.clone(),
+                        value.clone()
+                    ))
+                }
+            }
+            return Ok(vec);
+        }
+
+        // If the value is a resourcearray, check if it is inside
+        let val_equals = |val: &str| {
+            let q = q_value.unwrap();
+            val == q || {
+                if val.starts_with('[') {
+                    match crate::parse::parse_json_array(val) {
+                        Ok(vec) => return vec.contains(&q.into()),
+                        Err(_) => return val == q,
+                    }
+                }
+                false
+            }
+        };
+
+        // Find atoms matching the TPF query in a single resource
+        let mut find_in_resource = |resource: &Resource| {
+            let subj = resource.get_subject();
+            for (prop, val) in resource.get_propvals().iter() {
+                if hasprop && q_property.as_ref().unwrap() == prop {
+                    if hasval {
+                        if val_equals(&val.to_string()) {
+                            vec.push(Atom::new(subj.into(), prop.into(), val.clone()))
+                        }
+                        break;
+                    } else {
+                        vec.push(Atom::new(subj.into(), prop.into(), val.clone()))
+                    }
+                    break;
+                } else if hasval && !hasprop && val_equals(&val.to_string()) {
+                    vec.push(Atom::new(subj.into(), prop.into(), val.clone()))
+                }
+            }
+        };
+
+        match q_subject {
+            Some(sub) => match self.get_resource(&sub) {
+                Ok(resource) => {
+                    if hasprop | hasval {
+                        find_in_resource(&resource);
+                        Ok(vec)
+                    } else {
+                        resource.to_atoms()
+                    }
+                }
+                Err(_) => Ok(vec),
+            },
+            None => {
+                if hasval {
+                    let spm = self.get_prop_subject_map(q_value.unwrap())?;
+                    if hasprop {
+                        if let Some(set) = spm.get(q_property.unwrap())  {
+                            let base = self.get_base_url();
+                            for subj in set {
+                                if !include_external && !subj.starts_with(base) {
+                                    continue;
+                                }
+                                let property_full = self.get_property(q_property.unwrap())?;
+                                let mut datatype = property_full.data_type;
+                                // The value index stores only single subjects, not arrays.
+                                // However, this also means that it is not possible to find the actual _value_ of a thing
+                                // So for arrays, we simply return AtomicURLs.
+                                if datatype == DataType::ResourceArray {
+                                    datatype = DataType::AtomicUrl
+                                }
+                                let atom = Atom::new(subj.into(), q_property.unwrap().into(), Value::new(q_value.unwrap(), &datatype)?);
+                                vec.push(atom);
+                            }
+                        }
+                    } else {
+                        for (prop, set) in spm.iter() {
+                            for subj in set {
+                                let property_full = self.get_property(prop)?;
+                                let atom = Atom::new(subj.into(), prop.into(), Value::new(q_value.unwrap(), &property_full.data_type)?);
+                                vec.push(atom);
+                            }
+                        }
+                    }
+                    return Ok(vec)
+                }
+                // TODO: Add an index for searching only by property
+                for resource in self.all_resources(include_external) {
+                    find_in_resource(&resource);
+                }
+                Ok(vec)
+            }
+        }
+    }
 }
 
 fn corrupt_db_message(subject: &str) -> String {
@@ -258,7 +452,9 @@ fn corrupt_db_message(subject: &str) -> String {
 const DB_CORRUPT_MSG: &str = "Could not deserialize item from database. DB is possibly corrupt, could be due to an update or a lack of migrations. Restore to a previous version, export / serialize your data and import your data again.";
 
 #[cfg(test)]
-mod test {
+pub mod test {
+    use crate::urls;
+
     use super::*;
     use ntest::timeout;
 
@@ -268,31 +464,17 @@ mod test {
         let tmp_dir_path = "tmp/db";
         let _try_remove_existing = std::fs::remove_dir_all(tmp_dir_path);
         let store = Db::init(tmp_dir_path, "https://localhost".into()).unwrap();
-        store.populate().unwrap();
         let agent = store.create_agent(None).unwrap();
         store.set_default_agent(agent);
+        store.populate().unwrap();
         store
     }
 
-    /// TODO: find bug!
-    /// For some reason, this one keeps going on forever.
-    /// It calles create_agent before populating, and keeps requesting stuff.
-    fn _init_faulty() -> Db {
-        let tmp_dir_path = "tmp/db";
-        let _try_remove_existing = std::fs::remove_dir_all(tmp_dir_path);
-        let store = Db::init(tmp_dir_path, "https://localhost".into()).unwrap();
-        let agent = store.create_agent(None).unwrap();
-        store.populate().unwrap();
-        store.set_default_agent(agent);
-        store
-    }
-
-
-    /// Share the Db instance between tests. Otherwise, all tests try to init the same location on disk an throw errors.
+    /// Share the Db instance between tests. Otherwise, all tests try to init the same location on disk and throw errors.
     use lazy_static::lazy_static; // 1.4.0
     use std::sync::Mutex;
     lazy_static! {
-        static ref DB: Mutex<Db> = Mutex::new(init());
+        pub static ref DB: Mutex<Db> = Mutex::new(init());
     }
 
     #[test]
@@ -343,7 +525,8 @@ mod test {
     #[test]
     fn populate_collections() {
         let store = DB.lock().unwrap().clone();
-        println!("{:?}", store.all_resources(false));
+        let subjects: Vec<String> = store.all_resources(false).into_iter().map(|r| r.get_subject().into()).collect();
+        println!("{:?}", subjects);
         let collections_collection_url = format!("{}/collections", store.get_base_url());
         let my_resource = store
             .get_resource_extended(&collections_collection_url)
@@ -352,6 +535,25 @@ mod test {
             .get(crate::urls::COLLECTION_MEMBER_COUNT)
             .unwrap();
         println!("My value: {}", my_value);
-        assert_eq!(my_value.to_string(), "11");
+        assert_eq!(my_value.to_int().unwrap(), 11);
+    }
+
+    #[test]
+    /// Check if the cache is working
+    fn add_atom_to_index() {
+        let store = DB.lock().unwrap().clone();
+        let subject = urls::CLASS.into();
+        let property: String = urls::PARENT.into();
+        let val_string= urls::AGENT;
+        let value = Value::new(val_string, &DataType::AtomicUrl).unwrap();
+        // This atom should normally not exist - Agent is not the parent of Class.
+        let atom = Atom::new(subject, property.clone(), value);
+        store.add_atom_to_index(&atom).unwrap();
+        let found_no_external = store.tpf(None, Some(&property), Some(val_string), false).unwrap();
+        // Don't find the atom if no_external is true.
+        assert_eq!(found_no_external.len(), 0);
+        let found_external = store.tpf(None, Some(&property), Some(val_string), true).unwrap();
+        // If we see the atom, it's in the index.
+        assert_eq!(found_external.len(), 1);
     }
 }
