@@ -262,12 +262,12 @@ impl Resource {
             self.set_propval_unsafe(property, value)
         } else {
             Err(format!("Datatype for subject '{}', property '{}', value '{}' did not match. Wanted '{}', got '{}'",
-            self.get_subject(),
-            property,
-            value.to_string(),
-            required_datatype,
-            value.datatype()
-        ).into())
+                self.get_subject(),
+                property,
+                value.to_string(),
+                required_datatype,
+                value.datatype()
+            ).into())
         }
     }
 
@@ -300,6 +300,7 @@ impl Resource {
     /// TODO: Handle these changes in Commits
     /// introduce 'move' command? https://github.com/joepio/atomic/issues/44
     pub fn set_subject(&mut self, url: String) {
+        self.commit.set_subject(url.clone());
         self.subject = url;
     }
 
